@@ -1,4 +1,4 @@
-/* $Id: color.c 4328 2008-09-21 23:02:30Z astyanax $ */
+/* $Id: color.c 4367 2009-02-07 14:48:30Z astyanax $ */
 /**************************************************************************
  *   color.c                                                              *
  *                                                                        *
@@ -105,11 +105,13 @@ void color_init(void)
 /* Update the color information based on the current filename. */
 void color_update(void)
 {
-    const syntaxtype *tmpsyntax;
+    syntaxtype *tmpsyntax;
+    syntaxtype *defsyntax = NULL;
     colortype *tmpcolor, *defcolor = NULL;
 
     assert(openfile != NULL);
 
+    openfile->syntax = NULL;
     openfile->colorstrings = NULL;
 
     /* If we specified a syntax override string, use it. */
@@ -121,8 +123,10 @@ void color_update(void)
 
 	for (tmpsyntax = syntaxes; tmpsyntax != NULL;
 		tmpsyntax = tmpsyntax->next) {
-	    if (strcmp(tmpsyntax->desc, syntaxstr) == 0)
+	    if (strcmp(tmpsyntax->desc, syntaxstr) == 0) {
+		openfile->syntax = tmpsyntax;
 		openfile->colorstrings = tmpsyntax->color;
+	    }
 
 	    if (openfile->colorstrings != NULL)
 		break;
@@ -141,6 +145,7 @@ void color_update(void)
 	     * extensions, which we've checked for elsewhere.  Skip over
 	     * it here, but keep track of its color regexes. */
 	    if (strcmp(tmpsyntax->desc, "default") == 0) {
+		defsyntax = tmpsyntax;
 		defcolor = tmpsyntax->color;
 		continue;
 	    }
@@ -159,8 +164,10 @@ void color_update(void)
 		/* Set colorstrings if we matched the extension
 		 * regex. */
 		if (regexec(e->ext, openfile->filename, 0, NULL,
-			0) == 0)
+			0) == 0) {
+		    openfile->syntax = tmpsyntax;
 		    openfile->colorstrings = tmpsyntax->color;
+		}
 
 		if (openfile->colorstrings != NULL)
 		    break;
@@ -200,8 +207,10 @@ void color_update(void)
 #ifdef DEBUG
 		fprintf(stderr, "Comparing header regex \"%s\" to fileage \"%s\"...\n", e->ext_regex, openfile->fileage->data);
 #endif
-		    if (regexec(e->ext, openfile->fileage->data, 0, NULL, 0) == 0)
+		    if (regexec(e->ext, openfile->fileage->data, 0, NULL, 0) == 0) {
+			openfile->syntax = tmpsyntax;
 			openfile->colorstrings = tmpsyntax->color;
+		    }
 
 		    if (openfile->colorstrings != NULL)
 			break;
@@ -221,8 +230,10 @@ void color_update(void)
 
     /* If we didn't get a syntax based on the file extension, and we
      * have a default syntax, use it. */
-    if (openfile->colorstrings == NULL && defcolor != NULL)
+    if (openfile->colorstrings == NULL && defcolor != NULL) {
+	openfile->syntax = defsyntax;
 	openfile->colorstrings = defcolor;
+    }
 
     for (tmpcolor = openfile->colorstrings; tmpcolor != NULL;
 	tmpcolor = tmpcolor->next) {
@@ -243,4 +254,71 @@ void color_update(void)
     }
 }
 
+/* Reset the multicolor info cache for records for any lines which need
+   to be recalculated */
+void reset_multis_after(filestruct *fileptr, int mindex)
+{
+    filestruct *oof;
+    for (oof = fileptr->next; oof != NULL; oof = oof->next) {
+	alloc_multidata_if_needed(oof);
+	if (oof->multidata == NULL)
+	    continue;
+	if (oof->multidata[mindex] != CNONE)
+	    oof->multidata[mindex] = -1;
+	else
+	    break;
+    }
+}
+
+void reset_multis_before(filestruct *fileptr, int mindex)
+{
+    filestruct *oof;
+    for (oof = fileptr->prev; oof != NULL; oof = oof->prev) {
+	alloc_multidata_if_needed(oof);
+	if (oof->multidata == NULL)
+	    continue;
+	if (oof->multidata[mindex] != CNONE)
+	    oof->multidata[mindex] = -1;
+	else
+	    break;
+    }
+}
+
+
+/* Reset multi line strings around a filestruct ptr, trying to be smart about stopping */
+void reset_multis(filestruct *fileptr)
+{
+    int nobegin, noend;
+    regmatch_t startmatch, endmatch;
+    const colortype *tmpcolor = openfile->colorstrings;
+
+    if (!openfile->syntax)
+	return;
+
+    for (; tmpcolor != NULL; tmpcolor = tmpcolor->next) {
+
+	/* If it's not a multi-line regex, amscray */
+	if (tmpcolor->end == NULL)
+	    continue;
+
+	alloc_multidata_if_needed(fileptr);
+	/* Figure out where the first begin and end are to determine if
+	   things changed drastically for the precalculated multi values */
+        nobegin = regexec(tmpcolor->start, fileptr->data, 1, &startmatch, 0);
+        noend = regexec(tmpcolor->end, fileptr->data, 1, &endmatch, 0);
+	if (fileptr->multidata[tmpcolor->id] ==  CWHOLELINE) {
+	    if (nobegin && noend)
+		continue;
+	} else if (fileptr->multidata[tmpcolor->id] & CBEGINBEFORE && !noend
+	  && (nobegin || endmatch.rm_eo > startmatch.rm_eo)) {
+	    reset_multis_after(fileptr, tmpcolor->id);
+	    continue;
+	}
+
+	/* If we got here assume the worst */
+	reset_multis_before(fileptr, tmpcolor->id);
+	reset_multis_after(fileptr, tmpcolor->id);
+	fileptr->multidata[tmpcolor->id] = -1;
+    }
+}
 #endif /* ENABLE_COLOR */
