@@ -1,4 +1,4 @@
-/* $Id: files.c 4345 2008-10-22 22:11:46Z astyanax $ */
+/* $Id: files.c 4362 2009-02-03 05:05:58Z astyanax $ */
 /**************************************************************************
  *   files.c                                                              *
  *                                                                        *
@@ -311,6 +311,10 @@ filestruct *read_line(char *buf, filestruct *prevnode, bool
 	fileptr->data[buf_len - 1] = '\0';
 #endif
 
+#ifdef ENABLE_COLOR
+	fileptr->multidata = NULL;
+#endif
+
     if (*first_line_ins) {
 	/* Special case: We're inserting with the cursor on the first
 	 * line. */
@@ -600,7 +604,7 @@ void read_file(FILE *f, const char *filename, bool undoable)
  * return value.  *f is set to the opened file. */
 int open_file(const char *filename, bool newfie, FILE **f)
 {
-    struct stat fileinfo;
+    struct stat fileinfo, fileinfo2;
     int fd;
     char *full_filename;
 
@@ -609,10 +613,21 @@ int open_file(const char *filename, bool newfie, FILE **f)
     /* Get the specified file's full path. */
     full_filename = get_full_path(filename);
 
-    if (full_filename == NULL)
+    /* Okay, if we can't stat the path due to a component's
+       permissions, just try the relative one */
+    if (full_filename == NULL 
+	|| (stat(full_filename, &fileinfo) == -1 && stat(filename, &fileinfo2) != -1))
 	full_filename = mallocstrcpy(NULL, filename);
 
     if (stat(full_filename, &fileinfo) == -1) {
+	/* Well, maybe we can open the file even if the OS
+	   says its not there */
+        if ((fd = open(filename, O_RDONLY)) != -1) {
+	    statusbar(_("Reading File"));
+	    free(full_filename);
+	    return 0;
+	}
+
 	if (newfie) {
 	    statusbar(_("New File"));
 	    return -2;
@@ -780,14 +795,14 @@ void do_insertfile(
 #ifndef NANO_TINY
 #ifdef ENABLE_MULTIBUFFER
 
-	    if (s && s->scfunc == (void *) new_buffer_msg) {
+	    if (s && s->scfunc == NEW_BUFFER_MSG) {
 		/* Don't allow toggling if we're in view mode. */
 		if (!ISSET(VIEW_MODE))
 		    TOGGLE(MULTIBUFFER);
 		continue;
 	    } else
 #endif
-	    if (s && s->scfunc == (void *) ext_cmd_msg) {
+	    if (s && s->scfunc == EXT_CMD_MSG) {
 		execute = !execute;
 		continue;
 	    }
@@ -797,7 +812,7 @@ void do_insertfile(
 #endif /* !NANO_TINY */
 
 #ifndef DISABLE_BROWSER
-	    if (s && s->scfunc == (void *) to_files_msg) {
+	    if (s && s->scfunc == TO_FILES_MSG) {
 		char *tmp = do_browse_from(answer);
 
 		if (tmp == NULL)
@@ -1917,7 +1932,7 @@ bool do_writeout(bool exiting)
             s = get_shortcut(currmenu, &i, &meta_key, &func_key);
 
 #ifndef DISABLE_BROWSER
-	    if (s && s->scfunc == (void *) to_files_msg) {
+	    if (s && s->scfunc == TO_FILES_MSG) {
 		char *tmp = do_browse_from(answer);
 
 		if (tmp == NULL)
@@ -1929,26 +1944,26 @@ bool do_writeout(bool exiting)
 	    } else
 #endif /* !DISABLE_BROWSER */
 #ifndef NANO_TINY
-	    if (s && s->scfunc == (void *) dos_format_msg) {
+	    if (s && s->scfunc ==  DOS_FORMAT_MSG) {
 		openfile->fmt = (openfile->fmt == DOS_FILE) ? NIX_FILE :
 			DOS_FILE;
 		continue;
-	    } else if (s && s->scfunc == (void *) mac_format_msg) {
+	    } else if (s && s->scfunc ==  MAC_FORMAT_MSG) {
 		openfile->fmt = (openfile->fmt == MAC_FILE) ? NIX_FILE :
 			MAC_FILE;
 		continue;
-	    } else if (s && s->scfunc == (void *) backup_file_msg) {
+	    } else if (s && s->scfunc ==  BACKUP_FILE_MSG) {
 		TOGGLE(BACKUP_FILE);
 		continue;
 	    } else
 #endif /* !NANO_TINY */
-	    if (s && s->scfunc == (void *) prepend_msg) {
+	    if (s && s->scfunc ==  PREPEND_MSG) {
 		append = (append == PREPEND) ? OVERWRITE : PREPEND;
 		continue;
-	    } else if (s && s->scfunc == (void *) append_msg) {
+	    } else if (s && s->scfunc ==  APPEND_MSG) {
 		append = (append == APPEND) ? OVERWRITE : APPEND;
 		continue;
-	    } else if (s && s->scfunc == do_help_void) {
+	    } else if (s && s->scfunc == DO_HELP_VOID) {
 		continue;
 	    }
 
