@@ -1,4 +1,4 @@
-/* $Id: nano.c 4326 2008-09-06 06:52:47Z astyanax $ */ 
+/* $Id: nano.c 4364 2009-02-06 03:41:02Z astyanax $ */ 
 /**************************************************************************
  *   nano.c                                                               *
  *                                                                        *
@@ -32,6 +32,7 @@
 #include <errno.h>
 #include <ctype.h>
 #include <locale.h>
+#include <time.h>
 #ifdef ENABLE_UTF8
 #include <langinfo.h>
 #endif
@@ -67,6 +68,10 @@ filestruct *make_new_node(filestruct *prevnode)
     newnode->next = NULL;
     newnode->lineno = (prevnode != NULL) ? prevnode->lineno + 1 : 1;
 
+#ifdef ENABLE_COLOR
+    newnode->multidata = NULL;
+#endif
+
     return newnode;
 }
 
@@ -83,6 +88,9 @@ filestruct *copy_node(const filestruct *src)
     dst->next = src->next;
     dst->prev = src->prev;
     dst->lineno = src->lineno;
+#ifdef ENABLE_COLOR
+    dst->multidata = NULL;
+#endif
 
     return dst;
 }
@@ -119,6 +127,11 @@ void delete_node(filestruct *fileptr)
     if (fileptr->data != NULL)
 	free(fileptr->data);
 
+#ifdef ENABLE_COLOR
+    if (fileptr->multidata)
+	free(fileptr->multidata);
+#endif
+
     free(fileptr);
 }
 
@@ -356,6 +369,10 @@ void move_to_filestruct(filestruct **file_top, filestruct **file_bot,
     openfile->fileage->data = mallocstrcpy(NULL, "");
     openfile->filebot = openfile->fileage;
 
+#ifdef ENABLE_COLOR
+    openfile->fileage->multidata = NULL;
+#endif
+
     /* Restore the current line and cursor position.  If the mark begins
      * inside the partition, set the beginning of the mark to where the
      * saved text used to start. */
@@ -1019,6 +1036,75 @@ void do_exit(void)
     display_main_list();
 }
 
+
+
+static struct sigaction pager_oldaction, pager_newaction;  /* Original and temporary handlers for SIGINT. */
+static bool pager_sig_failed = FALSE; /* Did sigaction() fail without changing the signal handlers? */
+
+/* Things which need to be run regardless of whether
+   we finished the stdin pipe correctly or not */
+void finish_stdin_pager(void)
+{
+    FILE *f;
+    int ttystdin;
+
+    /* Read whatever we did get from stdin */
+    f = fopen("/dev/stdin", "rb");
+       if (f == NULL)
+        nperror("fopen");
+
+    read_file(f, "stdin", TRUE);
+    ttystdin = open("/dev/tty", O_RDONLY);
+    if (!ttystdin)
+        die(_("Couldn't reopen stdin from keyboard, sorry\n"));
+
+    dup2(ttystdin,0);
+    close(ttystdin);
+    tcgetattr(0, &oldterm);
+    if (!pager_sig_failed && sigaction(SIGINT, &pager_oldaction, NULL) == -1)
+        nperror("sigaction");
+    terminal_init();
+    doupdate();
+}
+
+
+/* Cancel reading from stdin like a pager */
+RETSIGTYPE cancel_stdin_pager(int signal)
+{
+    /* Currently do nothing, just handle the intr silently */
+}
+
+/* Let nano read stdin for the first file at least */
+void stdin_pager(void)
+{
+    endwin();
+    tcsetattr(0, TCSANOW, &oldterm);
+    fprintf(stderr, _("Reading from stdin, ^C to abort\n"));
+
+    /* Set things up so that Ctrl-C will cancel the new process. */
+    /* Enable interpretation of the special control keys so that we get
+     * SIGINT when Ctrl-C is pressed. */
+#ifndef NANO_TINY
+    enable_signals();
+#endif
+
+    if (sigaction(SIGINT, NULL, &pager_newaction) == -1) {
+	pager_sig_failed = TRUE;
+	nperror("sigaction");
+    } else {
+	pager_newaction.sa_handler = cancel_stdin_pager;
+	if (sigaction(SIGINT, &pager_newaction, &pager_oldaction) == -1) {
+	    pager_sig_failed = TRUE;
+	    nperror("sigaction");
+	}
+    }
+ 
+    open_buffer("", FALSE);
+    finish_stdin_pager();
+}
+
+
+
 /* Initialize the signal handlers. */
 void signal_init(void)
 {
@@ -1439,6 +1525,7 @@ int do_input(bool *meta_key, bool *func_key, bool *s_or_t, bool
 		kbinput = (int *)nrealloc(kbinput, kbinput_len *
 			sizeof(int));
 		kbinput[kbinput_len - 1] = input;
+
 	    }
 	}
 
@@ -1453,7 +1540,7 @@ int do_input(bool *meta_key, bool *func_key, bool *s_or_t, bool
 	     * for verbatim input, turn off prepending of wrapped
 	     * text. */
 	    if (have_shortcut && (!have_shortcut || s == NULL || s->scfunc !=
-		do_verbatim_input))
+		DO_VERBATIM_INPUT))
 		wrap_reset();
 #endif
 
@@ -1488,27 +1575,40 @@ int do_input(bool *meta_key, bool *func_key, bool *s_or_t, bool
 		default:
 		    /* If the function associated with this shortcut is
 		     * cutting or copying text, indicate this. */
-		    if (s->scfunc == do_cut_text_void
+		    if (s->scfunc == DO_CUT_TEXT_VOID
 #ifndef NANO_TINY
-			|| s->scfunc == do_copy_text || s->scfunc ==
-			do_cut_till_end
+			|| s->scfunc == DO_COPY_TEXT || s->scfunc ==
+			DO_CUT_TILL_END
 #endif
 			)
 			cut_copy = TRUE;
 
-		    if (s->scfunc != NULL) {
+		    if (s->scfunc != 0) {
 			const subnfunc *f = sctofunc((sc *) s);
 			*ran_func = TRUE;
 			if (ISSET(VIEW_MODE) && f && !f->viewok)
 			    print_view_warning();
 			else
 #ifndef NANO_TINY
-			    if (s->scfunc == (void *) do_toggle)
+			    if (s->scfunc ==  DO_TOGGLE)
 				do_toggle(s->toggle);
-			    else
+			    else {
 #endif
-				s->scfunc();
+				iso_me_harder_funcmap(s->scfunc);
+#ifdef ENABLE_COLOR
+				if (!f->viewok && openfile->syntax != NULL 
+					&& openfile->current->multidata && openfile->syntax->nmultis > 0) {
+				    reset_multis(openfile->current);
+				}
+				if (edit_refresh_needed) {
+				    edit_refresh();
+				    edit_refresh_needed = FALSE;
+				}
+#endif
+			    }
+#ifndef NANO_TINY
 		    }
+#endif
 		    *finished = TRUE;
 		    break;
 	    }
@@ -1546,7 +1646,7 @@ int do_mouse(void)
 	bool sameline;
 	    /* Did they click on the line with the cursor?  If they
 	     * clicked on the cursor, we set the mark. */
-	const filestruct *current_save = openfile->current;
+	filestruct *current_save = openfile->current;
 	size_t current_x_save = openfile->current_x;
 	size_t pww_save = openfile->placewewant;
 
@@ -1579,6 +1679,94 @@ int do_mouse(void)
 }
 #endif /* !DISABLE_MOUSE */
 
+#ifdef ENABLE_COLOR
+void alloc_multidata_if_needed(filestruct *fileptr)
+{
+    if (!fileptr->multidata)
+	fileptr->multidata = nmalloc(openfile->syntax->nmultis * sizeof(short));
+}
+
+/* Precalculate the multi-line start and end regex info so we can speed up
+   rendering (with any hope at all...) */
+void precalc_multicolorinfo(void)
+{
+    if (openfile->colorstrings != NULL && !ISSET(NO_COLOR_SYNTAX)) {
+	const colortype *tmpcolor = openfile->colorstrings;
+	regmatch_t startmatch, endmatch;
+	filestruct *fileptr, *endptr;
+	time_t last_check = time(NULL), cur_check = 0;
+
+	/* Let us get keypresses to see if the user is trying to
+	   start editing.  We may want to throw up a statusbar
+	   message before starting this later if it takes
+	   too long to do this routine.  For now silently
+	   abort if they hit a key */
+ 	nodelay(edit, FALSE);
+
+	for (; tmpcolor != NULL; tmpcolor = tmpcolor->next) {
+
+	    /* If it's not a multi-line regex, amscray */
+	    if (tmpcolor->end == NULL)
+		continue;
+
+	    for (fileptr = openfile->fileage; fileptr != NULL; fileptr = fileptr->next) {
+		int startx = 0;
+
+		alloc_multidata_if_needed(fileptr);
+
+		if ((cur_check = time(NULL)) - last_check > 1) {
+		    last_check = cur_check;
+		    if (wgetch(edit) != ERR)
+	   		goto precalc_cleanup;
+		}
+
+		fileptr->multidata[tmpcolor->id] = CNONE;
+		while (regexec(tmpcolor->start, &fileptr->data[startx], 1, &startmatch, 0)  == 0) {
+		    /* Look for end and start marking how many lines are encompassed
+		       whcih should speed up rendering later */
+		     startx += startmatch.rm_eo;
+
+		    /* Look on this line first for end */
+		    if (regexec(tmpcolor->end, &fileptr->data[startx], 1, &endmatch, 0)  == 0) {
+			startx += endmatch.rm_eo;
+			fileptr->multidata[tmpcolor->id] |= CSTARTENDHERE;
+			continue;
+		    }
+
+		    /* Nice, we didn't find the end regex on this line.  Let's start looking for it */
+		    for (endptr = fileptr->next; endptr != NULL; endptr = endptr->next) {
+
+			/* Check for keyboard input  again */
+			if ((cur_check = time(NULL)) - last_check > 1) {
+			    last_check = cur_check;
+			    if (wgetch(edit) != ERR)
+		   		goto precalc_cleanup;
+			}
+			if (regexec(tmpcolor->end, &endptr->data[startx], 1, &endmatch, 0) == 0)
+			   break;
+		    }
+
+		    if (endptr == NULL)
+			break;
+
+		    /* We found it, we found it, la la la la la.  Mark all the
+			lines in between and the ends properly */
+		    fileptr->multidata[tmpcolor->id] |= CENDAFTER;
+		    for (fileptr = fileptr->next; fileptr != endptr; fileptr = fileptr->next) {
+			alloc_multidata_if_needed(fileptr);
+			fileptr->multidata[tmpcolor->id] = CWHOLELINE;
+		    }
+		    alloc_multidata_if_needed(endptr);
+		    endptr->multidata[tmpcolor->id] |= CBEGINBEFORE;
+		}
+	    }
+	}
+    }
+precalc_cleanup:
+    nodelay(edit, FALSE);
+}
+#endif /* ENABLE_COLOR */
+
 /* The user typed output_len multibyte characters.  Add them to the edit
  * buffer, filtering out all ASCII control characters if allow_cntrls is
  * TRUE. */
@@ -1683,9 +1871,14 @@ void do_output(char *output, size_t output_len, bool allow_cntrls)
 
     openfile->placewewant = xplustabs();
 
-    if (do_refresh)
+
+#ifdef ENABLE_COLOR
+    reset_multis(openfile->current);
+#endif
+    if (do_refresh) {
 	edit_refresh();
-    else
+	edit_refresh_needed = FALSE;
+    } else
 	update_line(openfile->current, openfile->current_x);
 }
 
@@ -2212,6 +2405,12 @@ int main(int argc, char **argv)
 	optind++;
     }
 
+    if (optind < argc && !strcmp(argv[optind], "-")) {
+	stdin_pager();
+	set_modified();
+	optind++;
+    }
+
 #ifdef ENABLE_MULTIBUFFER
     old_multibuffer = ISSET(MULTIBUFFER);
     SET(MULTIBUFFER);
@@ -2267,6 +2466,11 @@ int main(int argc, char **argv)
     fprintf(stderr, "Main: top and bottom win\n");
 #endif
 
+#ifdef ENABLE_COLOR
+    if (openfile->syntax && openfile->syntax->nmultis > 0)
+	precalc_multicolorinfo();
+#endif
+
     if (startline > 1 || startcol > 1)
 	do_gotolinecolumn(startline, startcol, FALSE, FALSE, FALSE,
 		FALSE);
@@ -2313,3 +2517,4 @@ int main(int argc, char **argv)
     /* We should never get here. */
     assert(FALSE);
 }
+
