@@ -1,4 +1,4 @@
-/* $Id: nano.h 4340 2008-10-14 04:34:56Z astyanax $ */
+/* $Id: nano.h 4367 2009-02-07 14:48:30Z astyanax $ */
 /**************************************************************************
  *   nano.h                                                               *
  *                                                                        *
@@ -173,36 +173,6 @@ typedef enum {
     ADD, DEL, REPLACE, SPLIT, UNSPLIT, CUT, UNCUT, INSERT, OTHER
 } undo_type;
 
-/* Structure types. */
-typedef struct filestruct {
-    char *data;
-	/* The text of this line. */
-    ssize_t lineno;
-	/* The number of this line. */
-    struct filestruct *next;
-	/* Next node. */
-    struct filestruct *prev;
-	/* Previous node. */
-} filestruct;
-
-typedef struct partition {
-    filestruct *fileage;
-	/* The top line of this portion of the file. */
-    filestruct *top_prev;
-	/* The line before the top line of this portion of the file. */
-    char *top_data;
-	/* The text before the beginning of the top line of this portion
-	 * of the file. */
-    filestruct *filebot;
-	/* The bottom line of this portion of the file. */
-    filestruct *bot_next;
-	/* The line after the bottom line of this portion of the
-	 * file. */
-    char *bot_data;
-	/* The text after the end of the bottom line of this portion of
-	 * the file. */
-} partition;
-
 #ifdef ENABLE_COLOR
 typedef struct colortype {
     short fg;
@@ -226,6 +196,8 @@ typedef struct colortype {
 	/* The compiled end (if any) of the regex string. */
     struct colortype *next;
 	/* Next set of colors. */
+     int id;
+	/* basic id for assigning to lines later */
 } colortype;
 
 typedef struct exttype {
@@ -246,11 +218,61 @@ typedef struct syntaxtype {
 	/* Regexes to match on the 'header' (1st line) of the file */
     colortype *color;
 	/* The colors used in this syntax. */
+    int nmultis;
+	/* How many multi line strings this syntax has */
     struct syntaxtype *next;
 	/* Next syntax. */
 } syntaxtype;
+
+#define CNONE 		(1<<1)
+	/* Yay, regex doesn't apply to this line at all! */
+#define CBEGINBEFORE 	(1<<2)
+	/* regex starts on an earlier line, ends on this one */
+#define CENDAFTER 	(1<<3)
+	/* regex sraers on this line and ends on a later one */
+#define CWHOLELINE 	(1<<4)
+	/* whole line engulfed by the regex  start < me, end > me */
+#define CSTARTENDHERE 	(1<<5)
+	/* regex starts and ends within this line */
+#define CWTF		(1<<6)
+	/* Something else */
+
 #endif /* ENABLE_COLOR */
 
+
+/* Structure types. */
+typedef struct filestruct {
+    char *data;
+	/* The text of this line. */
+    ssize_t lineno;
+	/* The number of this line. */
+    struct filestruct *next;
+	/* Next node. */
+    struct filestruct *prev;
+	/* Previous node. */
+#ifdef ENABLE_COLOR
+    short *multidata;		/* Array of which multi-line regexes apply to this line */
+#endif
+} filestruct;
+
+typedef struct partition {
+    filestruct *fileage;
+	/* The top line of this portion of the file. */
+    filestruct *top_prev;
+	/* The line before the top line of this portion of the file. */
+    char *top_data;
+	/* The text before the beginning of the top line of this portion
+	 * of the file. */
+    filestruct *filebot;
+	/* The bottom line of this portion of the file. */
+    filestruct *bot_next;
+	/* The line after the bottom line of this portion of the
+	 * file. */
+    char *bot_data;
+	/* The text after the end of the bottom line of this portion of
+	 * the file. */
+} partition;
+
 #ifndef NANO_TINY
 typedef struct undo {
     ssize_t lineno;
@@ -323,6 +345,8 @@ typedef struct openfilestruct {
     undo_type last_action;
 #endif
 #ifdef ENABLE_COLOR
+    syntaxtype *syntax;
+	/* The  syntax struct for this file, if any */
     colortype *colorstrings;
 	/* The current file's associated colors. */
 #endif
@@ -380,7 +404,7 @@ typedef struct sc {
         /* The actual sequence to check on the the type is determined */
     int menu;
         /* What list does this apply to */
-    void (*scfunc)(void);
+    short scfunc;
         /* The function we're going to run */
     int toggle;
         /* If a toggle, what we're toggling */
@@ -392,7 +416,7 @@ typedef struct sc {
 } sc;
 
 typedef struct subnfunc {
-   void (*scfunc)(void);
+    short scfunc;
 	/* What function is this */
     int menus;
 	/* In what menus does this function applu */
@@ -696,8 +720,167 @@ typedef struct subnfunc {
 #define UNDO_DEL_DEL		(1<<0)
 #define UNDO_DEL_BACKSPACE	(1<<1)
 
+/* Since in ISO C you can't pass around function pointers anymore,
+  let's make some integer macros for function names, and then I
+  can go cut my wrists after writing the big switch statement
+  that will necessitate. */
+
 #endif /* !NANO_TINY */
 
+#define CASE_SENS_MSG 1
+#define BACKWARDS_MSG 2
+#define REGEXP_MSG 3
+#define WHEREIS_NEXT_MSG 4
+#define FIRST_FILE_MSG 5
+#define LAST_FILE_MSG 6
+#define GOTO_DIR_MSG 7
+#define TOTAL_REFRESH 8
+#define DO_HELP_VOID 9
+#define DO_SEARCH 10
+#define DO_PAGE_UP 11
+#define DO_PAGE_DOWN 12
+#define DO_UP_VOID 13
+#define DO_LEFT 14
+#define DO_DOWN_VOID 15
+#define DO_RIGHT 16
+#define DO_ENTER 17
+#define DO_EXIT 18
+#define NEW_BUFFER_MSG 19
+#define EXT_CMD_MSG 20
+#define TO_FILES_MSG 21
+#define DOS_FORMAT_MSG 23
+#define MAC_FORMAT_MSG 24
+#define BACKUP_FILE_MSG 25
+#define PREPEND_MSG 26
+#define APPEND_MSG 27
+#define DO_FIRST_LINE 28
+#define DO_LAST_LINE 29
+#define DO_TOGGLE 30
+#define GOTOTEXT_MSG 31
+#define NO_REPLACE_MSG 32
+#define DO_BACKSPACE 33
+#define DO_DELETE 34
+#define DO_TAB 35
+#define DO_VERBATIM_INPUT 36
+#define SWITCH_TO_NEXT_BUFFER_VOID 37
+#define SWITCH_TO_PREV_BUFFER_VOID 38
+#define DO_END 39
+#define DO_HOME 40
+#define NEXT_HISTORY_MSG 41
+#define PREV_HISTORY_MSG 42
+#define DO_REDO 43
+#define DO_UNDO 44
+#define DO_WORDLINECHAR_COUNT 45
+#define DO_FIND_BRACKET 46
+#define DO_PREV_WORD_VOID 47
+#define DO_SUSPEND_VOID 48
+#define CANCEL_MSG 49
+#define DO_WRITEOUT_VOID 50
+#define DO_INSERTFILE_VOID 51
+#define DO_CUT_TEXT_VOID 52
+#define DO_UNCUT_TEXT 53
+#define DO_CURSORPOS_VOID 54
+#define DO_GOTOLINECOLUMN_VOID 55
+#define DO_REPLACE 56
+#define DO_JUSTIFY_VOID 57
+#define DO_PARA_BEGIN_VOID 58
+#define DO_PARA_END_VOID 59
+#define DO_FULL_JUSTIFY 60
+#define DO_MARK 61
+#define DO_RESEARCH 62
+#define DO_COPY_TEXT 63
+#define DO_INDENT_VOID 64
+#define DO_UNINDENT 65
+#define DO_SCROLL_UP 66
+#define DO_SCROLL_DOWN 67
+#define DO_NEXT_WORD_VOID 68
+#define DO_CUT_TILL_END 69
+#define NANO_GOTODIR_MSG 70
+#define NANO_LASTFILE_MSG 71
+#define NANO_FIRSTFILE_MSG 72
+#define INSERT_FILE_MSG 73
+#define NANO_MULTIBUFFER_MSG 74
+#define NANO_EXECUTE_MSG 75
+#define NANO_BACKUP_MSG 76
+#define NANO_PREPEND_MSG 77
+#define NANO_APPEND_MSG 78
+#define NANO_MAC_MSG 79
+#define NANO_DOS_MSG 80
+#define NANO_TOFILES_MSG 81
+#define NANO_NEXT_HISTORY_MSG 82
+#define NANO_PREV_HISTORY_MSG 83
+#define NANO_REGEXP_MSG 84
+#define NANO_REVERSE_MSG 85
+#define NANO_CASE_MSG 86
+#define NANO_SUSPEND_MSG 87
+#define SUSPEND_MSG 88
+#define NANO_REFRESH_MSG 89
+#define REFRESH_MSG 90
+#define NANO_WORDCOUNT_MSG 91
+#define NANO_FULLJUSTIFY_MSG 92
+#define FULLJSTIFY_MSG 93
+#define XOFF_COMPLAINT 94
+#define XON_COMPLAINT 95
+#define NANO_CUT_TILL_END_MSG 96
+#define NANO_BACKSPACE_MSG 97
+#define NANO_DELETE_MSG 98
+#define NANO_ENTER_MSG 99
+#define NANO_TAB_MSG 100
+#define NANO_VERBATIM_MSG 101
+#define NANO_NEXTFILE_MSG 102
+#define NANO_PREVFILE_MSG 103
+#define NANO_SCROLLDOWN_MSG 104
+#define NANO_SCROLLUP_MSG 105
+#define NANO_BRACKET_MSG 106
+#define NANO_PARAEND_MSG 107
+#define END_OF_PAR_MSG 108
+#define NANO_PARABEGIN_MSG 109
+#define BEG_OF_PAR_MSG 110
+#define NANO_END_MSG 111
+#define NANO_HOME_MSG 112
+#define NANO_NEXTLINE_MSG 113
+#define NANO_PREVLINE_MSG 114
+#define NANO_PREVWORD_MSG 115
+#define NANO_NEXTWORD_MSG 116
+#define NANO_BACK_MSG 117
+#define NANO_FORWARD_MSG 118
+#define NANO_REDO_MSG 119
+#define NANO_UNDO_MSG 120
+#define NANO_UNINDENT_MSG 121
+#define NANO_INDENT_MSG 122
+#define NANO_COPY_MSG 123
+#define NANO_WHEREIS_NEXT_MSG 124
+#define NANO_MARK_MSG 125
+#define NANO_REPLACE_MSG 126
+#define REPLACE_MSG 127
+#define NANO_GOTOLINE_MSG 128
+#define NANO_LASTLINE_MSG 129
+#define NANO_FIRSTLINE_MSG 130
+#define NANO_SPELL_MSG 131
+#define DO_SPELL 132
+#define NANO_CURSORPOS_MSG 133
+#define NANO_UNCUT_MSG 134
+#define GET_HELP_MSG 135
+#define NANO_HELP_MSG 136
+#define NANO_CANCEL_MSG 137
+#define NANO_EXIT_MSG 138
+#define EXIT_MSG 139
+#define NANO_EXITBROWSER_MSG 140
+#define NANO_WRITEOUT_MSG 141
+#define NANO_DISABLED_MSG 142
+#define NANO_INSERT_MSG 143
+#define WHEREIS_MSG 144
+#define NANO_WHEREIS_MSG 145
+#define NANO_PREVPAGE_MSG 146
+#define NANO_NEXTPAGE_MSG 147
+#define NANO_CUT_MSG 148
+#define DO_CUT_TEXT 149
+#define DO_NEXT_WORD 150
+#define DO_PREV_WORD 151
+
+
+
+
 #define VIEW TRUE
 #define NOVIEW FALSE
 
