@@ -1,4 +1,4 @@
-/* $Id: proto.h 4317 2008-08-30 21:00:00Z astyanax $ */
+/* $Id: proto.h 4367 2009-02-07 14:48:30Z astyanax $ */
 /**************************************************************************
  *   proto.h                                                              *
  *                                                                        *
@@ -76,7 +76,7 @@ extern char *quoteerr;
 extern size_t quotelen;
 #endif
 #endif
-
+bool nodelay_mode;
 extern char *answer;
 
 extern ssize_t tabsize;
@@ -100,6 +100,7 @@ extern syntaxtype *syntaxes;
 extern char *syntaxstr;
 #endif
 
+extern bool edit_refresh_needed;
 extern const shortcut *currshortcut;
 extern int currmenu;
 
@@ -542,6 +543,8 @@ void parse_syntax(char *ptr);
 void parse_include(char *ptr);
 short color_to_short(const char *colorname, bool *bright);
 void parse_colors(char *ptr, bool icase);
+void reset_multis(filestruct *fileptr);
+void alloc_multidata_if_needed(filestruct *fileptr);
 #endif
 void parse_rcfile(FILE *rcstream
 #ifdef ENABLE_COLOR
@@ -743,7 +746,7 @@ int get_mouseinput(int *mouse_x, int *mouse_y, bool allow_shortcuts);
 #endif
 const sc *get_shortcut(int menu, int *kbinput, bool
 	*meta_key, bool *func_key);
-const sc *first_sc_for(int menu, void *func);
+const sc *first_sc_for(int menu, short func);
 void blank_line(WINDOW *win, int y, int x, int n);
 void blank_titlebar(void);
 void blank_topbar(void);
@@ -759,13 +762,13 @@ void statusbar(const char *msg, ...);
 void bottombars(int menu);
 void onekey(const char *keystroke, const char *desc, size_t len);
 void reset_cursor(void);
-void edit_draw(const filestruct *fileptr, const char *converted, int
+void edit_draw(filestruct *fileptr, const char *converted, int
 	line, size_t start);
-void update_line(const filestruct *fileptr, size_t index);
+void update_line(filestruct *fileptr, size_t index);
 bool need_horizontal_update(size_t pww_save);
 bool need_vertical_update(size_t pww_save);
 void edit_scroll(scroll_dir direction, ssize_t nlines);
-void edit_redraw(const filestruct *old_current, size_t pww_save);
+void edit_redraw(filestruct *old_current, size_t pww_save);
 void edit_refresh(void);
 void edit_update(update_type location);
 void total_redraw(void);
@@ -776,6 +779,7 @@ void do_cursorpos_void(void);
 void do_replace_highlight(bool highlight, const char *word);
 char *flagtostr(int flag);
 const subnfunc *sctofunc(sc *s);
+const subnfunc *getfuncfromkey(WINDOW *win);
 void print_sclist(void);
 sc *strtosc(int menu, char *input);
 function_type strtokeytype(char *str);
@@ -783,7 +787,7 @@ int strtomenu(char *input);
 void assign_keyinfo(sc *s);
 void xon_complaint(void);
 void xoff_complaint(void);
-int sc_seq_or (void *func, int defaultval);
+int sc_seq_or (short func, int defaultval);
 void do_suspend_void(void);
 
 const char *cancel_msg;
@@ -814,6 +818,9 @@ const char *prepend_msg;
 const char *backup_file_msg;
 const char *gototext_msg;
 const char *new_buffer_msg;
+void iso_me_harder_funcmap(short func);
+void enable_nodelay(void);
+void disable_nodelay(void);
 
 #ifdef HAVE_REGEX_H
 const char *regexp_msg;
