@@ -1,4 +1,4 @@
-/* $Id: rcfile.c 4342 2008-10-14 19:55:34Z astyanax $ */
+/* $Id: rcfile.c 4365 2009-02-07 00:01:40Z astyanax $ */
 /**************************************************************************
  *   rcfile.c                                                             *
  *                                                                        *
@@ -107,6 +107,7 @@ static exttype *endheader = NULL;
 	/* End of header list */
 static colortype *endcolor = NULL;
 	/* The end of the color list for the current syntax. */
+
 #endif
 
 /* We have an error in some part of the rcfile.  Print the error message
@@ -296,6 +297,7 @@ void parse_syntax(char *ptr)
     endsyntax->extensions = NULL;
     endsyntax->headers = NULL;
     endsyntax->next = NULL;
+    endsyntax->nmultis = 0;
 
 #ifdef DEBUG
     fprintf(stderr, "Starting a new syntax type: \"%s\"\n", nameptr);
@@ -427,7 +429,7 @@ void parse_keybinding(char *ptr)
 
 #ifdef DEBUG
     fprintf(stderr, "newsc now address %d, menu func assigned = %d, menu = %d\n",
-	&newsc, &newsc->scfunc, menu);
+	&newsc, newsc->scfunc, menu);
 #endif
 
 
@@ -468,7 +470,7 @@ void parse_include(char *ptr)
 {
     struct stat rcinfo;
     FILE *rcstream;
-    char *option, *full_option, *nanorc_save = nanorc;
+    char *option, *nanorc_save = nanorc;
     size_t lineno_save = lineno;
 
     option = ptr;
@@ -476,37 +478,33 @@ void parse_include(char *ptr)
 	option++;
     ptr = parse_argument(ptr);
 
-    /* Get the specified file's full path. */
-    full_option = get_full_path(option);
-
-    if (full_option == NULL)
-	full_option = mallocstrcpy(NULL, option);
+    /* Can't get the specified file's full path cause it may screw up
+	our cwd depending on the parent dirs' permissions, (see Savannah bug 25297) */
 
     /* Don't open directories, character files, or block files. */
-    if (stat(full_option, &rcinfo) != -1) {
+    if (stat(option, &rcinfo) != -1) {
 	if (S_ISDIR(rcinfo.st_mode) || S_ISCHR(rcinfo.st_mode) ||
 		S_ISBLK(rcinfo.st_mode)) {
 	    rcfile_error(S_ISDIR(rcinfo.st_mode) ?
 		_("\"%s\" is a directory") :
 		_("\"%s\" is a device file"), option);
-	    goto cleanup_include;
 	}
     }
 
     /* Open the new syntax file. */
-    if ((rcstream = fopen(full_option, "rb")) == NULL) {
+    if ((rcstream = fopen(option, "rb")) == NULL) {
 	rcfile_error(_("Error reading %s: %s"), option,
 		strerror(errno));
-	goto cleanup_include;
+	return;
     }
 
     /* Use the name and line number position of the new syntax file
      * while parsing it, so we can know where any errors in it are. */
-    nanorc = full_option;
+    nanorc = option;
     lineno = 0;
 
 #ifdef DEBUG
-    fprintf(stderr, "Parsing file \"%s\"\n", full_option);
+    fprintf(stderr, "Parsing file \"%s\"\n", option);
 #endif
 
     parse_rcfile(rcstream
@@ -520,8 +518,6 @@ void parse_include(char *ptr)
     nanorc = nanorc_save;
     lineno = lineno_save;
 
-  cleanup_include:
-    free(full_option);
 }
 
 /* Return the short value corresponding to the color named in colorname,
@@ -716,6 +712,10 @@ void parse_colors(char *ptr, bool icase)
 	    /* Save the ending regex string if it's valid. */
 	    newcolor->end_regex = (nregcomp(fgstr, icase ? REG_ICASE :
 		0)) ? mallocstrcpy(NULL, fgstr) : NULL;
+
+	    /* Lame way to skip another static counter */
+            newcolor->id = endsyntax->nmultis;
+            endsyntax->nmultis++;
 	}
     }
 }
@@ -791,7 +791,7 @@ static void check_vitals_mapped(void)
     subnfunc *f;
     int v;
 #define VITALS 5
-    void *vitals[VITALS] = { do_exit, do_exit, (void *) cancel_msg, (void *) cancel_msg, (void *) cancel_msg };
+    short vitals[VITALS] = { DO_EXIT, DO_EXIT, CANCEL_MSG, CANCEL_MSG, CANCEL_MSG };
     int inmenus[VITALS] = { MMAIN, MHELP, MWHEREIS, MREPLACE, MGOTOLINE };
 
     for  (v = 0; v < VITALS; v++) {
