@@ -1,4 +1,4 @@
-/* $Id: search.c 4343 2008-10-15 01:25:25Z astyanax $ */
+/* $Id: search.c 4350 2009-01-24 22:40:41Z astyanax $ */
 /**************************************************************************
  *   search.c                                                             *
  *                                                                        *
@@ -28,6 +28,7 @@
 #include <unistd.h>
 #include <ctype.h>
 #include <errno.h>
+#include <time.h>
 
 static bool search_last_line = FALSE;
 	/* Have we gone past the last line while searching? */
@@ -137,7 +138,7 @@ int search_init(bool replacing, bool use_answer)
     int i = 0;
     char *buf;
     sc *s;
-    void *func = NULL;
+    char func = 0;
     bool meta_key = FALSE, func_key = FALSE;
     static char *backupstring = NULL;
 	/* The search string we'll be using. */
@@ -235,26 +236,26 @@ int search_init(bool replacing, bool use_answer)
 #endif
 		;
 #ifndef NANO_TINY
-	} else if (func == (void *) case_sens_msg) {
+	} else if (func == CASE_SENS_MSG) {
 		TOGGLE(CASE_SENSITIVE);
 		backupstring = mallocstrcpy(backupstring, answer);
 		return 1;
-	} else if (func == (void *) backwards_msg) {
+	} else if (func == BACKWARDS_MSG) {
 		TOGGLE(BACKWARDS_SEARCH);
 		backupstring = mallocstrcpy(backupstring, answer);
 		return 1;
 #endif
 #ifdef HAVE_REGEX_H
-	} else if (func == (void *) regexp_msg) {
+	} else if (func == REGEXP_MSG) {
 		TOGGLE(USE_REGEXP);
 		backupstring = mallocstrcpy(backupstring, answer);
 		return 1;
 #endif
-	} else if (func == (void *) do_replace || 
-	  func == (void *) no_replace_msg) {
+	} else if (func == DO_REPLACE || 
+	  func == NO_REPLACE_MSG) {
 		backupstring = mallocstrcpy(backupstring, answer);
 		return -2;	/* Call the opposite search function. */
-	} else if (func == do_gotolinecolumn_void) {
+	} else if (func == DO_GOTOLINECOLUMN_VOID) {
 		do_gotolinecolumn(openfile->current->lineno,
 			openfile->placewewant + 1, TRUE, TRUE, FALSE,
 			TRUE);
@@ -288,6 +289,8 @@ bool findnextstr(
     ssize_t current_y_find = openfile->current_y;
     filestruct *fileptr = openfile->current;
     const char *rev_start = fileptr->data, *found = NULL;
+    const subnfunc *f;
+    time_t lastkbcheck = time(NULL);
 
     /* rev_start might end up 1 character before the start or after the
      * end of the line.  This won't be a problem because strstrwrapper()
@@ -302,7 +305,17 @@ bool findnextstr(
 	openfile->current_x + 1;
 
     /* Look for needle in the current line we're searching. */
+    enable_nodelay();
     while (TRUE) {
+        if (time(NULL) - lastkbcheck > 1) {
+            lastkbcheck = time(NULL);
+	    f = getfuncfromkey(edit);
+            if (f && f->scfunc == CANCEL_MSG) {
+		statusbar(_("Cancelled"));
+		return FALSE;
+	    }
+	}
+
 	found = strstrwrapper(fileptr->data, needle, rev_start);
 
 	/* We've found a potential match. */
@@ -348,6 +361,7 @@ bool findnextstr(
 	/* We've finished processing the file, so get out. */
 	if (search_last_line) {
 	    not_found_msg(needle);
+            disable_nodelay();
 	    return FALSE;
 	}
 
@@ -405,9 +419,11 @@ bool findnextstr(
 #endif
 	) {
 	not_found_msg(needle);
+	disable_nodelay();
 	return FALSE;
     }
 
+    disable_nodelay();
     /* We've definitely found something. */
     openfile->current = fileptr;
     openfile->current_x = current_x_find;
@@ -1021,7 +1037,7 @@ void do_gotolinecolumn(ssize_t line, ssize_t column, bool use_answer,
 
 
 	s = get_shortcut(currmenu, &i, &meta_key, &func_key);
-	if (s && s->scfunc == (void *) gototext_msg) {
+	if (s && s->scfunc ==  GOTOTEXT_MSG) {
 	    /* Keep answer up on the statusbar. */
 	    search_init(TRUE, TRUE);
 
